from .classes import Severity, Output, Format

class Settings():
    """Class to set logging parameters"""
    def __init__(self):
        self.message = ""
        self.app_name = "justlog"
        self.current_log_level = Severity.INF
        self.colorized_logs = False
        self.log_format = Format.TEXT
        self.log_output = []
        self.log_file = f"/tmp/{self.app_name}"
        self.fields = {}
        self.string_format = ""
        self.timestamp_format = "%Y-%m-%d %X.%f"
        self.tcp_output_host = ""
        self.tcp_output_port = 5000
        self.http_url = ""
        self.http_headers = {}
        self.http_print_response = False
    def update_field(self, key, value):
        self.fields.update({key:value})
    def delete_field(self, key):
        del self.fields[key]
    def log_level_name(self):
        return self.current_log_level.name



import syslog
import sys
import socket
import requests
from colorama import init, Fore
from .settings import Settings
from .classes import Severity, Output, Format
from .formatter import json_formatter, text_formatter

init()

class Logger(Settings):
    def __init__(self, settings):
        self.settings = settings

    def log(self, message):
        self.settings.message = message
        if not isinstance(self.settings.log_format, Format):
            raise TypeError(f"Unsupported or unrecognized format: {type(self.settings.log_format)}")
        for output in self.settings.log_output:
            if not isinstance(output, Output):
                raise TypeError(f"Unsupported or unrecognized output: {type(output)}")
        if not isinstance(self.settings.current_log_level, Severity):
            raise TypeError(f"Unsupported or unrecognized severity: {type(self.settings.current_log_level)}")
        if self.settings.log_format == Format.JSON:
            self.settings = json_formatter(self.settings)
        if self.settings.log_format == Format.TEXT:
            self.settings = text_formatter(self.settings)
        if Output.STDOUT in self.settings.log_output:
            log_to_stdout(self.settings)
        if Output.STDERR in self.settings.log_output:
            log_to_stderr(self.settings)
        if Output.FILE in self.settings.log_output:
            log_to_file(self.settings.message, self.settings.log_file)
        if Output.SYSLOG in self.settings.log_output:
            log_to_sys(self.settings.message, self.settings.current_log_level)
        if Output.TCP in self.settings.log_output:
            log_to_tcp(self.settings.message, self.settings)
        if Output.HTTP in self.settings.log_output:
            log_to_http(self.settings.message, self.settings)
        self.settings.message = ""
    def debug(self, message):
        self.settings.current_log_level = Severity.DBG
        self.log(message)
    def info(self, message):
        self.settings.current_log_level = Severity.INF
        self.log(message)
    def warning(self, message):
        self.settings.current_log_level = Severity.WRN
        self.log(message)
    def error(self, message):
        self.settings.current_log_level = Severity.ERR
        self.log(message)

def log_to_stdout(settings: Settings):
    reset = Fore.RESET
    color = Fore.WHITE
    if settings.colorized_logs:
        if settings.current_log_level == Severity.WRN:
            color = Fore.YELLOW
        if settings.current_log_level == Severity.ERR:
            color = Fore.RED
    print(f"{color}{settings.message}{reset}")

def log_to_stderr(settings: Settings):
    reset = Fore.RESET
    color = Fore.WHITE
    if settings.colorized_logs:
        if settings.current_log_level == Severity.WRN:
            color = Fore.YELLOW
        if settings.current_log_level == Severity.ERR:
            color = Fore.RED
    print(f"{color}{settings.message}{reset}", file=sys.stderr)

def log_to_file(message, log_file):
    f = open(log_file, "a+")
    f.write(message + "\n")
    f.close()

def log_to_sys(message, severity):
    if severity == Severity.DBG:
        syslog.syslog(syslog.LOG_DEBUG, message)
    if severity == Severity.INF:
        syslog.syslog(syslog.LOG_INFO, message)
    if severity == Severity.WRN:
        syslog.syslog(syslog.LOG_WARNING, message)
    if severity == Severity.ERR:
        syslog.syslog(syslog.LOG_ERR, message)

def log_to_tcp(message, settings):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.connect((settings.tcp_output_host, settings.tcp_output_port))
        sock.sendall(bytes(message + "\n", "utf-8"))
        sock.close()

def log_to_http(message, settings):
    r = requests.post(settings.http_url, message, headers=settings.http_headers)
    if settings.http_print_response:
        print(r.content)



import json
import datetime
from .settings import Settings


def json_formatter(settings: Settings):
    settings.update_field("message", settings.message)
    settings.message = json.dumps(settings.fields)
    return builtins_formatter(settings)

def text_formatter(settings: Settings):
    buffer = settings.string_format
    settings.update_field("message", settings.message)
    for field in settings.fields:
        buffer = buffer.replace(f"${field}", settings.fields[field])
    settings.message = buffer
    return builtins_formatter(settings)

def builtins_formatter(settings: Settings):
    """Format values that need to be evaluated on every call."""
    buffer = settings.message.replace("$CURRENT_LOG_LEVEL", settings.log_level_name())
    buffer = buffer.replace("$TIMESTAMP", f"{datetime.datetime.now().strftime(settings.timestamp_format)}")
    settings.message = buffer
    return settings



from enum import IntEnum, Enum

class Severity(IntEnum):
    DBG = 1
    INF = 2
    WRN = 3
    ERR = 4

class Format(Enum):
    JSON = 0
    TEXT = 1

class Output(Enum):
    STDOUT = 0
    STDERR = 1
    FILE = 2
    SYSLOG = 3
    HTTP = 4
    TCP = 5



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="justlog",
    version="0.0.7",
    author="Gabisonfire",
    author_email="gabisonfire@github.com",
    description="A simple logging library for Python 3",
    keywords="logging json simple http tcp quick logs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.*',
    url="https://github.com/Gabisonfire/justlog",
    packages=["justlog"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'colorama',
          'requests'
      ]
)
